// ===== Fichier au format UTF-8 =====
/*globals document*/
// Les éléments modifiables du DOM
var dom_page, dom_titreExpr, dom_divExpr;

// supprime tous les fils d'un élément du DOM
function effacer(dom_element) {
	while (dom_element.firstChild) {
		dom_element.removeChild(dom_element.firstChild);
	}
}

// crée un élément du DOM : <balise class="classe">html</balise>
function noeud(balise, html, classe) {
	var element = document.createElement(balise);
	if (html) {
		element.innerHTML = html;
	}
	if (classe) {
		element.className = classe;
	}
	return element;
}

// crée un élément DOM qui représente une expression
//    contenant un champ de saisie de texte
//    <div class="expr">...</div>
function nouvelleExpressionEditable() {
	// code HTML : prompt <span> et champ de saisie <input>
	var caseDeSaisie = '<span class="message">&lt;nom&gt;[,&lt;arité&gt;] ? </span><input type="text"  onchange="valider(this)" />';
	// la <div> de classe "expr" qui représente l'expression
	var div = noeud('div', caseDeSaisie, 'expr');
	// chaque <div> d'une expression a un attribut "nom" 
	// qui la représente de façon lisible et qui sera
	// mis à jour dynamiquement au cours de la
	// définition de l'expression (au départ "•")
	div.setAttribute("nom", "•");
	return div;
}

// crée dans divPere une expression à arguments éditables
// 'divPere' est un élément <div> représantant une expression
// 'nom'     est le nom de la fonction à définir
// 'arite    est le nombre des arguments de cette fonction
// Si l'expression est une constante ou une variable: arite==0
// Sinon on ajoute après le nom un nombre de
//      nouvelleExpressionEditable() égal à arite
function creerExpressionDans(divPere, nom, arite) {
	var k;
	// on supprime la case de saisie
	effacer(divPere);
	// on y ajoute le texte du nom de la fonction
	divPere.appendChild(document.createTextNode(nom));
	// on définit l'attribut "nom" qui permettra de
	//  recalculer dynamiquement le titre de l'expression
	divPere.setAttribute("nom", nom);
	// on ajoute des cases de saisie selon l'arité
	for (k = 0; k < arite; k++) {
		divPere.appendChild(nouvelleExpressionEditable());
	}
}

//======================== contrôles =========================
// donne le focus à la première cases de saisie de texte
// c'est à dire que le curseur est placé dans le champ
function activerPremiereCase() {
	// la liste de toutes les cases de saisie disponibles 
	var cases = document.getElementsByTagName('input');
	if (cases.length > 0) {
		// redonne le focus à la première cases
		cases[0].focus();
	}
}

// Le titre de  l'expression est un texte qui représente
// l'expression sous forme fonctionnelle lisible humainement
// Ce texte est reconstruit dynamiquement lors de
// chaque modification de  l'expression.
function titreExpression(x) {
	var txt = "";
	// on ajoute d'abord le nom de la fonction
	// pris dans l'attribut "nom"
	txt += x.getAttribute("nom");
	// on récupère les éléments fils 
	var fils = x.childNodes;
	var args = "";
	var sep = "";
	for (var k = 0; k < fils.length; k ++) {
		// les fils sont :
		//   - soit des textes (noms de la fonction)
		//   - soit des <div> les arguments de la fonction
		if (fils[k].tagName === "DIV") {
			// on ajoute le titre de chaque argument
			args += sep + titreExpression(fils[k]);
			// séparés par des virgules
			sep = ", ";
		}
	}
	if (args.length > 0) {
		// si la liste des arguments n'est pas vide
		// on l'ajoute avec des parenthèses autour 
		txt += "(" + args + ")";
	}
	return txt;
}

// validation du contenu d'un champ de saisie de texte
// appel par une des cases de saisie actuellemnt affichées, 
// celle dont le contenu vien d'être validé.
//   'qui' est est l'élément du DOM ayant effectué l'appel:
//   <input type="text"  onchange="valider(this)" />
//   dont on récupère la valeur pour analyse
function valider(qui) {
	// récupératon du texte contenu dans le champ de saisie
	var reponse = qui.value;
	if (reponse) {  // si ce texte n'est pas vide
		// sépare les parties à gauche et à droite de la virgule
		reponse = reponse.split(',');
		// reponse[0] est le nom qui précède la virgule
		var nom = String(reponse[0]);
		if (nom.length === 0) {
			// si le nom est vide on efface le texte de la case
			// et on retourne sans rien faire de plus
			qui.value = "";
			return;
		}
		// reponse[1] est le nombre qui suit la virgule
		// si ce n'est pas un nombre c'est considéré comme 0
		var arite = Number(reponse[1]);
		// qui.parentNode : la <div> contenant la case de saisie
		// dans laquelle on va créer la nouvelle expression
		creerExpressionDans(qui.parentNode, nom, arite);
	}
	// on actualise le titre de l'expression
	dom_titreExpr.innerHTML = "Expression : &nbsp; ";
	// en fonction de la valeur actuelle de celle-ci
	dom_titreExpr.innerHTML += titreExpression(dom_divExpr);
	// place le curseur dans la nouvelle case de saisie de texte
	activerPremiereCase();
}

// ajoute une nouvelle expression vide à la page
function nouvelleExpression() {
	// paragraphe contenant la définition de l'expression
	// ce message sera modifié dynamiquement lors de la
	// création de l'expression par l'utilisateur
	dom_titreExpr = noeud("p", "Expression : &nbsp;  ?", "def");
	dom_page.appendChild(dom_titreExpr);
	// la <div> contenant l'expression éditable :
	// un champ de saisie de texte au départ
	dom_divExpr = nouvelleExpressionEditable();
	dom_page.appendChild(dom_divExpr);
	// place le curseur dans la case de saisie de texte
	activerPremiereCase();
}

// appel par le bouton qui efface l'expression créée
function effacerPage() {
	effacer(dom_page);
	nouvelleExpression();
}

//-------------- init --------------
function init() {
	dom_page = document.getElementById('page');
	nouvelleExpression();
}
